@@ -32,8 +32,9 @@
 #include "gsk/vulkan/gskvulkanrenderer.h"
 #endif
 
-#ifndef NODE_EDITOR_SOURCE_DIR
-#define NODE_EDITOR_SOURCE_DIR "." /* Fallback */
+#include <cairo.h>
+#ifdef CAIRO_HAS_SVG_SURFACE
+#include <cairo-svg.h>
 #endif
 
 typedef struct
@@ -57,6 +58,7 @@ struct _NodeEditorWindow
   GtkWidget *testcase_cairo_checkbutton;
   GtkWidget *testcase_name_entry;
   GtkWidget *testcase_save_button;
+  GtkWidget *scale_scale;
 
   GtkWidget *renderer_listbox;
   GListStore *renderers;
@@ -171,6 +173,8 @@ text_changed (GtkTextBuffer    *buffer,
   GBytes *bytes;
   GtkTextIter iter;
   GtkTextIter start, end;
+  float scale;
+  GskRenderNode *big_node;
 
   g_array_remove_range (self->errors, 0, self->errors->len);
   text = get_current_text (self->text_buffer);
@@ -181,6 +185,22 @@ text_changed (GtkTextBuffer    *buffer,
 
   /* If this is too slow, go fix the parser performance */
   self->node = gsk_render_node_deserialize (bytes, deserialize_error_func, self);
+
+  scale = gtk_scale_button_get_value (GTK_SCALE_BUTTON (self->scale_scale));
+  if (self->node && scale != 0.)
+    {
+      scale = pow (2., scale);
+      big_node = gsk_transform_node_new (self->node, gsk_transform_scale (NULL, scale, scale));
+    }
+  else if (self->node)
+    {
+      big_node = gsk_render_node_ref (self->node);
+    }
+  else
+    {
+      big_node = NULL;
+    }
+
   g_bytes_unref (bytes);
   if (self->node)
     {
@@ -190,18 +210,27 @@ text_changed (GtkTextBuffer    *buffer,
       graphene_rect_t bounds;
       guint i;
 
+      snapshot = gtk_snapshot_new ();
+      gsk_render_node_get_bounds (big_node, &bounds);
+      gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (- bounds.origin.x, - bounds.origin.y));
+      gtk_snapshot_append_node (snapshot, big_node);
+      paintable = gtk_snapshot_free_to_paintable (snapshot, &bounds.size);
+      gtk_picture_set_paintable (GTK_PICTURE (self->picture), paintable);
+      g_clear_object (&paintable);
+
       snapshot = gtk_snapshot_new ();
       gsk_render_node_get_bounds (self->node, &bounds);
       gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (- bounds.origin.x, - bounds.origin.y));
       gtk_snapshot_append_node (snapshot, self->node);
       paintable = gtk_snapshot_free_to_paintable (snapshot, &bounds.size);
-      gtk_picture_set_paintable (GTK_PICTURE (self->picture), paintable);
+
       for (i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (self->renderers)); i++)
         {
           gpointer item = g_list_model_get_item (G_LIST_MODEL (self->renderers), i);
           gtk_renderer_paintable_set_paintable (item, paintable);
           g_object_unref (item);
         }
+
       g_clear_object (&paintable);
     }
   else
@@ -209,6 +238,8 @@ text_changed (GtkTextBuffer    *buffer,
       gtk_picture_set_paintable (GTK_PICTURE (self->picture), NULL);
     }
 
+  g_clear_pointer (&big_node, gsk_render_node_unref);
+
   gtk_text_buffer_get_start_iter (self->text_buffer, &iter);
 
   while (!gtk_text_iter_is_end (&iter))
@@ -277,6 +308,14 @@ text_changed (GtkTextBuffer    *buffer,
                                      &start, &end);
 }
 
+static void
+scale_changed (GObject          *object,
+               GParamSpec       *pspec,
+               NodeEditorWindow *self)
+{
+  text_changed (self->text_buffer, self);
+}
+
 static gboolean
 text_view_query_tooltip_cb (GtkWidget        *widget,
                             int               x,
@@ -527,45 +566,36 @@ node_editor_window_load (NodeEditorWindow *self,
 }
 
 static void
-open_response_cb (GtkWidget        *dialog,
-                  int               response,
-                  NodeEditorWindow *self)
+open_response_cb (GObject *source,
+                  GAsyncResult *result,
+                  void *user_data)
 {
-  gtk_widget_hide (dialog);
+  GtkFileDialog *dialog = GTK_FILE_DIALOG (source);
+  NodeEditorWindow *self = user_data;
+  GFile *file;
 
-  if (response == GTK_RESPONSE_ACCEPT)
+  file = gtk_file_dialog_open_finish (dialog, result, NULL);
+  if (file)
     {
-      GFile *file;
-
-      file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
       node_editor_window_load (self, file);
       g_object_unref (file);
     }
-
-  gtk_window_destroy (GTK_WINDOW (dialog));
 }
 
 static void
 show_open_filechooser (NodeEditorWindow *self)
 {
-  GtkWidget *dialog;
+  GtkFileDialog *dialog;
+  GFile *cwd;
 
-  dialog = gtk_file_chooser_dialog_new ("Open node file",
-                                        GTK_WINDOW (self),
-                                        GTK_FILE_CHOOSER_ACTION_OPEN,
-                                        "_Cancel", GTK_RESPONSE_CANCEL,
-                                        "_Load", GTK_RESPONSE_ACCEPT,
-                                        NULL);
-
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-  GFile *cwd = g_file_new_for_path (".");
-  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), cwd, NULL);
+  dialog = gtk_file_dialog_new ();
+  gtk_file_dialog_set_title (dialog, "Open node file");
+  cwd = g_file_new_for_path (".");
+  gtk_file_dialog_set_initial_folder (dialog, cwd);
   g_object_unref (cwd);
-
-  g_signal_connect (dialog, "response", G_CALLBACK (open_response_cb), self);
-  gtk_widget_show (dialog);
+  gtk_file_dialog_open (dialog, GTK_WINDOW (self),
+                        NULL, open_response_cb, self);
+  g_object_unref (dialog);
 }
 
 static void
@@ -576,21 +606,22 @@ open_cb (GtkWidget        *button,
 }
 
 static void
-save_response_cb (GtkWidget        *dialog,
-                  int               response,
-                  NodeEditorWindow *self)
+save_response_cb (GObject *source,
+                  GAsyncResult *result,
+                  void *user_data)
 {
-  gtk_widget_hide (dialog);
+  GtkFileDialog *dialog = GTK_FILE_DIALOG (source);
+  NodeEditorWindow *self = user_data;
+  GFile *file;
 
-  if (response == GTK_RESPONSE_ACCEPT)
+  file = gtk_file_dialog_save_finish (dialog, result, NULL);
+  if (file)
     {
-      GFile *file;
       char *text;
       GError *error = NULL;
 
       text = get_current_text (self->text_buffer);
 
-      file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
       g_file_replace_contents (file, text, strlen (text),
                                NULL, FALSE,
                                G_FILE_CREATE_NONE,
@@ -599,68 +630,69 @@ save_response_cb (GtkWidget        *dialog,
                                &error);
       if (error != NULL)
         {
-          GtkWidget *message_dialog;
+          GtkAlertDialog *alert;
 
-          message_dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (self))),
-                                                   GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                   GTK_MESSAGE_INFO,
-                                                   GTK_BUTTONS_OK,
-                                                   "Saving failed");
-          gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message_dialog),
-                                                    "%s", error->message);
-          g_signal_connect (message_dialog, "response", G_CALLBACK (gtk_window_destroy), NULL);
-          gtk_widget_show (message_dialog);
+          alert = gtk_alert_dialog_new ("Saving failed");
+          gtk_alert_dialog_set_detail (alert, error->message);
+          gtk_alert_dialog_show (alert,
+                                 GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (self))));
+          g_object_unref (alert);
           g_error_free (error);
         }
 
       g_free (text);
       g_object_unref (file);
     }
-
-  gtk_window_destroy (GTK_WINDOW (dialog));
 }
 
 static void
 save_cb (GtkWidget        *button,
          NodeEditorWindow *self)
 {
-  GtkWidget *dialog;
+  GtkFileDialog *dialog;
+  GFile *cwd;
 
-  dialog = gtk_file_chooser_dialog_new ("Save node",
-                                        GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (button))),
-                                        GTK_FILE_CHOOSER_ACTION_SAVE,
-                                        "_Cancel", GTK_RESPONSE_CANCEL,
-                                        "_Save", GTK_RESPONSE_ACCEPT,
-                                        NULL);
-
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-  GFile *cwd = g_file_new_for_path (".");
-  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), cwd, NULL);
+  dialog = gtk_file_dialog_new ();
+  gtk_file_dialog_set_title (dialog, "Save node");
+  cwd = g_file_new_for_path (".");
+  gtk_file_dialog_set_initial_folder (dialog, cwd);
+  gtk_file_dialog_set_initial_name (dialog, "demo.node");
   g_object_unref (cwd);
-
-  g_signal_connect (dialog, "response", G_CALLBACK (save_response_cb), self);
-  gtk_widget_show (dialog);
+  gtk_file_dialog_save (dialog,
+                        GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (button))),
+                        NULL,
+                        save_response_cb, self);
+  g_object_unref (dialog);
 }
 
-static GdkTexture *
-create_texture (NodeEditorWindow *self)
+static GskRenderNode *
+create_node (NodeEditorWindow *self)
 {
   GdkPaintable *paintable;
   GtkSnapshot *snapshot;
-  GskRenderer *renderer;
   GskRenderNode *node;
-  GdkTexture *texture;
 
   paintable = gtk_picture_get_paintable (GTK_PICTURE (self->picture));
   if (paintable == NULL ||
       gdk_paintable_get_intrinsic_width (paintable) <= 0 ||
       gdk_paintable_get_intrinsic_height (paintable) <= 0)
     return NULL;
+
   snapshot = gtk_snapshot_new ();
   gdk_paintable_snapshot (paintable, snapshot, gdk_paintable_get_intrinsic_width (paintable), gdk_paintable_get_intrinsic_height (paintable));
   node = gtk_snapshot_free_to_node (snapshot);
+
+  return node;
+}
+
+static GdkTexture *
+create_texture (NodeEditorWindow *self)
+{
+  GskRenderer *renderer;
+  GskRenderNode *node;
+  GdkTexture *texture;
+
+  node = create_node (self);
   if (node == NULL)
     return NULL;
 
@@ -671,6 +703,58 @@ create_texture (NodeEditorWindow *self)
   return texture;
 }
 
+#ifdef CAIRO_HAS_SVG_SURFACE
+static cairo_status_t
+cairo_serializer_write (gpointer             user_data,
+                        const unsigned char *data,
+                        unsigned int         length)
+{
+  g_byte_array_append (user_data, data, length);
+
+  return CAIRO_STATUS_SUCCESS;
+}
+
+static GBytes *
+create_svg (GskRenderNode  *node,
+            GError        **error)
+{
+  cairo_surface_t *surface;
+  cairo_t *cr;
+  graphene_rect_t bounds;
+  GByteArray *array;
+
+  gsk_render_node_get_bounds (node, &bounds);
+  array = g_byte_array_new ();
+
+  surface = cairo_svg_surface_create_for_stream (cairo_serializer_write,
+                                                 array,
+                                                 bounds.size.width,
+                                                 bounds.size.height);
+  cairo_svg_surface_set_document_unit (surface, CAIRO_SVG_UNIT_PX);
+  cairo_surface_set_device_offset (surface, -bounds.origin.x, -bounds.origin.y);
+
+  cr = cairo_create (surface);
+  gsk_render_node_draw (node, cr);
+  cairo_destroy (cr);
+
+  cairo_surface_finish (surface);
+  if (cairo_surface_status (surface) == CAIRO_STATUS_SUCCESS)
+    {
+      cairo_surface_destroy (surface);
+      return g_byte_array_free_to_bytes (array);
+    }
+  else
+    {
+      g_set_error (error,
+                   G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "%s", cairo_status_to_string (cairo_surface_status (surface)));
+      cairo_surface_destroy (surface);
+      g_byte_array_unref (array);
+      return NULL;
+    }
+}
+#endif
+
 static GdkTexture *
 create_cairo_texture (NodeEditorWindow *self)
 {
@@ -703,61 +787,144 @@ create_cairo_texture (NodeEditorWindow *self)
 }
 
 static void
-export_image_response_cb (GtkWidget  *dialog,
-                          int         response,
-                          GdkTexture *texture)
+export_image_saved_cb (GObject      *source,
+                       GAsyncResult *result,
+                       void         *user_data)
 {
-  gtk_widget_hide (dialog);
+  GError *error = NULL;
 
-  if (response == GTK_RESPONSE_ACCEPT)
+  if (!g_file_replace_contents_finish (G_FILE (source), result, NULL, &error))
     {
-      GFile *file;
+      GtkAlertDialog *alert;
 
-      file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
-      if (!gdk_texture_save_to_png (texture, g_file_peek_path (file)))
-        {
-          GtkWidget *message_dialog;
+      alert = gtk_alert_dialog_new ("Exporting to image failed");
+      gtk_alert_dialog_set_detail (alert, error->message);
+      gtk_alert_dialog_show (alert, NULL);
+      g_object_unref (alert);
+      g_clear_error (&error);
+    }
+}
 
-          message_dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_window_get_transient_for (GTK_WINDOW (dialog))),
-                                                   GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                   GTK_MESSAGE_INFO,
-                                                   GTK_BUTTONS_OK,
-                                                   "Exporting to image failed");
-          g_signal_connect (message_dialog, "response", G_CALLBACK (gtk_window_destroy), NULL);
-          gtk_widget_show (message_dialog);
-        }
+static void
+export_image_response_cb (GObject      *source,
+                          GAsyncResult *result,
+                          void         *user_data)
+{
+  GtkFileDialog *dialog = GTK_FILE_DIALOG (source);
+  GskRenderNode *node = user_data;
+  GFile *file;
+  char *uri;
+  GBytes *bytes;
 
-      g_object_unref (file);
+  file = gtk_file_dialog_save_finish (dialog, result, NULL);
+  if (file == NULL)
+    {
+      gsk_render_node_unref (node);
+      return;
     }
 
-  gtk_window_destroy (GTK_WINDOW (dialog));
-  g_object_unref (texture);
+  uri = g_file_get_uri (file);
+#ifdef CAIRO_HAS_SVG_SURFACE
+  if (g_str_has_suffix (uri, "svg"))
+    {
+      GError *error = NULL;
+
+      bytes = create_svg (node, &error);
+      if (bytes == NULL)
+        {
+          GtkAlertDialog *alert;
+
+          alert = gtk_alert_dialog_new ("Exporting to image failed");
+          gtk_alert_dialog_set_detail (alert, error->message);
+          gtk_alert_dialog_show (alert, NULL);
+          g_object_unref (alert);
+          g_clear_error (&error);
+        }
+    }
+  else
+#endif
+    {
+      GdkTexture *texture;
+      GskRenderer *renderer;
+
+      renderer = gsk_gl_renderer_new ();
+      if (!gsk_renderer_realize (renderer, NULL, NULL))
+        {
+          g_object_unref (renderer);
+          renderer = gsk_cairo_renderer_new ();
+          if (!gsk_renderer_realize (renderer, NULL, NULL))
+            {
+              g_assert_not_reached ();
+            }
+        }
+      texture = gsk_renderer_render_texture (renderer, node, NULL);
+      gsk_renderer_unrealize (renderer);
+      g_object_unref (renderer);
+
+      if (g_str_has_suffix (uri, "tiff"))
+        bytes = gdk_texture_save_to_tiff_bytes (texture);
+      else
+        bytes = gdk_texture_save_to_png_bytes (texture);
+      g_object_unref (texture);
+    }
+  g_free (uri);
+
+  if (bytes)
+    {
+      g_file_replace_contents_bytes_async (file,
+                                           bytes,
+                                           NULL,
+                                           FALSE,
+                                           0,
+                                           NULL,
+                                           export_image_saved_cb,
+                                           NULL);
+      g_bytes_unref (bytes);
+    }
+  gsk_render_node_unref (node);
+  g_object_unref (file);
 }
 
 static void
 export_image_cb (GtkWidget        *button,
                  NodeEditorWindow *self)
 {
-  GdkTexture *texture;
-  GtkWidget *dialog;
+  GskRenderNode *node;
+  GtkFileDialog *dialog;
+  GtkFileFilter *filter;
+  GListStore *filters;
 
-  texture = create_texture (self);
-  if (texture == NULL)
+  node = create_node (self);
+  if (node == NULL)
     return;
 
-  dialog = gtk_file_chooser_dialog_new ("",
-                                        GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (button))),
-                                        GTK_FILE_CHOOSER_ACTION_SAVE,
-                                        "_Cancel", GTK_RESPONSE_CANCEL,
-                                        "_Save", GTK_RESPONSE_ACCEPT,
-                                        NULL);
+  filters = g_list_store_new (GTK_TYPE_FILE_FILTER);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_add_mime_type (filter, "image/png");
+  g_list_store_append (filters, filter);
+  g_object_unref (filter);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_add_mime_type (filter, "image/svg+xml");
+  g_list_store_append (filters, filter);
+  g_object_unref (filter);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_add_mime_type (filter, "image/tiff");
+  g_list_store_append (filters, filter);
+  g_object_unref (filter);
 
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-  g_signal_connect (dialog, "response", G_CALLBACK (export_image_response_cb), texture);
-  gtk_widget_show (dialog);
+  dialog = gtk_file_dialog_new ();
+  gtk_file_dialog_set_title (dialog, "");
+  gtk_file_dialog_set_initial_name (dialog, "example.png");
+  gtk_file_dialog_set_filters (dialog, G_LIST_MODEL (filters));
+  gtk_file_dialog_save (dialog,
+                        GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (button))),
+                        NULL,
+                        export_image_response_cb, node);
+  g_object_unref (filters);
+  g_object_unref (dialog);
 }
 
+
 static void
 clip_image_cb (GtkWidget        *button,
                NodeEditorWindow *self)
@@ -790,12 +957,52 @@ testcase_name_entry_changed_cb (GtkWidget        *button,
     gtk_widget_set_sensitive (self->testcase_save_button, FALSE);
 }
 
+/* Returns the location where gsk test cases are stored in
+ * the GTK testsuite, if we can determine it.
+ *
+ * When running node editor outside of a GTK build, you can
+ * set GTK_SOURCE_DIR to point it at the checkout.
+ */
+static char *
+get_source_dir (void)
+{
+  const char *subdir = "testsuite/gsk/compare";
+  const char *source_dir;
+  char *current_dir;
+  char *dir;
+
+  source_dir = g_getenv ("GTK_SOURCE_DIR");
+  current_dir = g_get_current_dir ();
+
+  if (source_dir)
+    {
+      char *abs_source_dir = g_canonicalize_filename (source_dir, NULL);
+      dir = g_canonicalize_filename (subdir, abs_source_dir);
+      g_free (abs_source_dir);
+    }
+  else
+    {
+      dir = g_canonicalize_filename (subdir, current_dir);
+    }
+
+  if (g_file_test (dir, G_FILE_TEST_EXISTS))
+    {
+      g_free (current_dir);
+
+      return dir;
+    }
+
+  g_free (dir);
+
+  return current_dir;
+}
+
 static void
 testcase_save_clicked_cb (GtkWidget        *button,
                           NodeEditorWindow *self)
 {
   const char *testcase_name = gtk_editable_get_text (GTK_EDITABLE (self->testcase_name_entry));
-  char *source_dir = g_canonicalize_filename (NODE_EDITOR_SOURCE_DIR, NULL);
+  char *source_dir = get_source_dir ();
   char *node_file_name;
   char *node_file;
   char *png_file_name;
@@ -808,6 +1015,8 @@ testcase_save_clicked_cb (GtkWidget        *button,
   node_file = g_build_filename (source_dir, node_file_name, NULL);
   g_free (node_file_name);
 
+  g_debug ("Saving testcase in %s", node_file);
+
   png_file_name = g_strconcat (testcase_name, ".png", NULL);
   png_file = g_build_filename (source_dir, png_file_name, NULL);
   g_free (png_file_name);
@@ -852,6 +1061,14 @@ dark_mode_cb (GtkToggleButton *button,
                 NULL);
 }
 
+static void
+node_editor_window_dispose (GObject *object)
+{
+  gtk_widget_dispose_template (GTK_WIDGET (object), NODE_EDITOR_WINDOW_TYPE);
+
+  G_OBJECT_CLASS (node_editor_window_parent_class)->dispose (object);
+}
+
 static void
 node_editor_window_finalize (GObject *object)
 {
@@ -946,6 +1163,7 @@ node_editor_window_class_init (NodeEditorWindowClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
 
+  object_class->dispose = node_editor_window_dispose;
   object_class->finalize = node_editor_window_finalize;
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -962,6 +1180,7 @@ node_editor_window_class_init (NodeEditorWindowClass *class)
   gtk_widget_class_bind_template_child (widget_class, NodeEditorWindow, testcase_cairo_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, NodeEditorWindow, testcase_name_entry);
   gtk_widget_class_bind_template_child (widget_class, NodeEditorWindow, testcase_save_button);
+  gtk_widget_class_bind_template_child (widget_class, NodeEditorWindow, scale_scale);
 
   gtk_widget_class_bind_template_callback (widget_class, text_view_query_tooltip_cb);
   gtk_widget_class_bind_template_callback (widget_class, open_cb);
@@ -1068,6 +1287,7 @@ node_editor_window_init (NodeEditorWindow *self)
 
   self->text_buffer = gtk_text_buffer_new (self->tag_table);
   g_signal_connect (self->text_buffer, "changed", G_CALLBACK (text_changed), self);
+  g_signal_connect (self->scale_scale, "notify::value", G_CALLBACK (scale_changed), self);
   gtk_text_view_set_buffer (GTK_TEXT_VIEW (self->text_view), self->text_buffer);
 
   /* Default */
@@ -1089,6 +1309,13 @@ node_editor_window_init (NodeEditorWindow *self)
          "  }\n"
          "  transform: translate(0, 140);\n"
          "}", -1);
+
+  if (g_getenv ("GSK_RENDERER"))
+    {
+      char *new_title = g_strdup_printf ("GTK Node Editor - %s", g_getenv ("GSK_RENDERER"));
+      gtk_window_set_title (GTK_WINDOW (self), new_title);
+      g_free (new_title);
+    }
 }
 
 NodeEditorWindow *
