@@ -39,6 +39,7 @@
 
 enum {
   PROP_0,
+  PROP_DESCRIPTION,
   PROP_DISPLAY,
   PROP_MANUFACTURER,
   PROP_MODEL,
@@ -81,6 +82,10 @@ gdk_monitor_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_DESCRIPTION:
+      g_value_set_string (value, monitor->description);
+      break;
+
     case PROP_DISPLAY:
       g_value_set_object (value, monitor->display);
       break;
@@ -154,6 +159,7 @@ gdk_monitor_finalize (GObject *object)
 {
   GdkMonitor *monitor = GDK_MONITOR (object);
 
+  g_free (monitor->description);
   g_free (monitor->connector);
   g_free (monitor->manufacturer);
   g_free (monitor->model);
@@ -170,15 +176,25 @@ gdk_monitor_class_init (GdkMonitorClass *class)
   object_class->get_property = gdk_monitor_get_property;
   object_class->set_property = gdk_monitor_set_property;
 
+  /**
+   * GdkMonitor:description: (attributes org.gtk.Property.get=gdk_monitor_get_description)
+   *
+   * A short description of the monitor, meant for display to the user.
+   *
+   * Since: 4.10
+   */
+  props[PROP_DESCRIPTION] =
+    g_param_spec_string ("description", NULL, NULL,
+                         NULL,
+                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
   /**
    * GdkMonitor:display: (attributes org.gtk.Property.get=gdk_monitor_get_display)
    *
    * The `GdkDisplay` of the monitor.
    */
   props[PROP_DISPLAY] =
-    g_param_spec_object ("display",
-                         "Display",
-                         "The display of the monitor",
+    g_param_spec_object ("display", NULL, NULL,
                          GDK_TYPE_DISPLAY,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
@@ -188,9 +204,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The manufacturer name.
    */
   props[PROP_MANUFACTURER] =
-    g_param_spec_string ("manufacturer",
-                         "Manufacturer",
-                         "The manufacturer name",
+    g_param_spec_string ("manufacturer", NULL, NULL,
                          NULL,
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
@@ -200,9 +214,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The model name.
    */
   props[PROP_MODEL] =
-    g_param_spec_string ("model",
-                         "Model",
-                         "The model name",
+    g_param_spec_string ("model", NULL, NULL,
                          NULL,
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
@@ -212,9 +224,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The connector name.
    */
   props[PROP_CONNECTOR] =
-    g_param_spec_string ("connector",
-                         "Connector",
-                         "The connector name",
+    g_param_spec_string ("connector", NULL, NULL,
                          NULL,
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
@@ -224,9 +234,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The scale factor.
    */
   props[PROP_SCALE_FACTOR] =
-    g_param_spec_int ("scale-factor",
-                      "Scale factor",
-                      "The scale factor",
+    g_param_spec_int ("scale-factor", NULL, NULL,
                       0, G_MAXINT,
                       1,
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
@@ -237,9 +245,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The geometry of the monitor.
    */
   props[PROP_GEOMETRY] =
-    g_param_spec_boxed ("geometry",
-                        "Geometry",
-                        "The geometry of the monitor",
+    g_param_spec_boxed ("geometry", NULL, NULL,
                         GDK_TYPE_RECTANGLE,
                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
@@ -249,9 +255,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The width of the monitor, in millimeters.
    */
   props[PROP_WIDTH_MM] =
-    g_param_spec_int ("width-mm",
-                      "Physical width",
-                      "The width of the monitor, in millimeters",
+    g_param_spec_int ("width-mm", NULL, NULL,
                       0, G_MAXINT,
                       0,
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
@@ -262,9 +266,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The height of the monitor, in millimeters.
    */
   props[PROP_HEIGHT_MM] =
-    g_param_spec_int ("height-mm",
-                      "Physical height",
-                      "The height of the monitor, in millimeters",
+    g_param_spec_int ("height-mm", NULL, NULL,
                       0, G_MAXINT,
                       0,
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
@@ -275,9 +277,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The refresh rate, in milli-Hertz.
    */
   props[PROP_REFRESH_RATE] =
-    g_param_spec_int ("refresh-rate",
-                      "Refresh rate",
-                      "The refresh rate, in millihertz",
+    g_param_spec_int ("refresh-rate", NULL, NULL,
                       0, G_MAXINT,
                       0,
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
@@ -288,9 +288,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * The subpixel layout.
    */
   props[PROP_SUBPIXEL_LAYOUT] =
-    g_param_spec_enum ("subpixel-layout",
-                       "Subpixel layout",
-                       "The subpixel layout",
+    g_param_spec_enum ("subpixel-layout", NULL, NULL,
                        GDK_TYPE_SUBPIXEL_LAYOUT,
                        GDK_SUBPIXEL_LAYOUT_UNKNOWN,
                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
@@ -301,9 +299,7 @@ gdk_monitor_class_init (GdkMonitorClass *class)
    * Whether the object is still valid.
    */
   props[PROP_VALID] =
-    g_param_spec_boolean ("valid",
-                          "Valid",
-                          "Whether the monitor is still valid",
+    g_param_spec_boolean ("valid", NULL, NULL,
                           TRUE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
@@ -399,6 +395,10 @@ gdk_monitor_get_height_mm (GdkMonitor *monitor)
  *
  * Gets the name of the monitor's connector, if available.
  *
+ * These are strings such as "eDP-1", or "HDMI-2". They depend
+ * on software and hardware configuration, and should not be
+ * relied on as stable identifiers of a specific monitor.
+ *
  * Returns: (transfer none) (nullable): the name of the connector
  */
 const char *
@@ -641,3 +641,33 @@ gdk_monitor_is_valid (GdkMonitor *monitor)
 
   return monitor->valid;
 }
+
+/**
+ * gdk_monitor_get_description: (attributes org.gtk.Method.get_property=description)
+ * @monitor: a `GdkMonitor`
+ *
+ * Gets a string describing the monitor, if available.
+ *
+ * This can be used to identify a monitor in the UI.
+ *
+ * Returns: (transfer none) (nullable): the monitor description
+ *
+ * Since: 4.10
+ */
+const char *
+gdk_monitor_get_description (GdkMonitor *monitor)
+{
+  g_return_val_if_fail (GDK_IS_MONITOR (monitor), NULL);
+
+  return monitor->description;
+}
+
+void
+gdk_monitor_set_description (GdkMonitor *monitor,
+                             const char *description)
+{
+  g_free (monitor->description);
+  monitor->description = g_strdup (description);
+  g_object_notify_by_pspec (G_OBJECT (monitor), props[PROP_DESCRIPTION]);
+}
+
