@@ -586,13 +586,6 @@ _gdk_wayland_display_open (const char *display_name)
 
   GDK_DEBUG (MISC, "opening display %s", display_name ? display_name : "");
 
-  /* If this variable is unset then wayland initialisation will surely
-   * fail, logging a fatal error in the process.  Save ourselves from
-   * that.
-   */
-  if (g_getenv ("XDG_RUNTIME_DIR") == NULL)
-    return NULL;
-
   wl_log_set_handler_client (log_handler);
 
   wl_display = wl_display_connect (display_name);
@@ -1082,6 +1075,9 @@ gdk_wayland_display_set_cursor_theme (GdkDisplay *display,
   g_assert (display_wayland);
   g_assert (display_wayland->shm);
 
+  if (size == 0)
+    size = 24;
+
   if (g_strcmp0 (name, display_wayland->cursor_theme_name) == 0 &&
       display_wayland->cursor_theme_size == size)
     return;
