@@ -17,34 +17,29 @@
  */
 
 #include "config.h"
-#define GDK_PIXBUF_ENABLE_BACKEND /* Ugly? */
 #include "gdkdisplay.h"
 #include "gdkcursor.h"
 #include "gdkwin32.h"
 #include "gdktextureprivate.h"
-#include "gdkintl.h"
 
 #include "gdkdisplay-win32.h"
 
-#ifdef __MINGW32__
-#include <w32api.h>
-#endif
-
 #include "xcursors.h"
 
-typedef struct _DefaultCursor {
+#include <stdint.h>
+
+static struct {
   char *name;
   char *id;
-} DefaultCursor;
-
-static DefaultCursor default_cursors[] = {
+} default_cursors[] = {
+  /* -- Win32 cursor names: -- */
   { "appstarting", IDC_APPSTARTING },
   { "arrow", IDC_ARROW },
   { "cross", IDC_CROSS },
   { "hand",  IDC_HAND },
   { "help",  IDC_HELP },
   { "ibeam", IDC_IBEAM },
-  /* an X cursor name, for compatibility with GTK: */
+  /* -- X11 cursor names: -- */
   { "left_ptr_watch", IDC_APPSTARTING },
   { "sizeall", IDC_SIZEALL },
   { "sizenesw", IDC_SIZENESW },
@@ -53,7 +48,7 @@ static DefaultCursor default_cursors[] = {
   { "sizewe", IDC_SIZEWE },
   { "uparrow", IDC_UPARROW },
   { "wait", IDC_WAIT },
-  /* css cursor names: */
+  /* -- CSS cursor names: -- */
   { "default", IDC_ARROW },
   { "pointer", IDC_HAND },
   { "progress", IDC_APPSTARTING },
@@ -93,18 +88,17 @@ struct _GdkWin32HCursor
 
   /* Do not do any modifications to the handle
    * (i.e. do not call DestroyCursor() on it).
-   * It's a "read-only" copy, the original is stored
-   * in the display instance.
-   */
+   * It's a "read-only" copy, the original is
+   * stored in the display instance */
   HANDLE           readonly_handle;
 
-  /* This is a way to access the real handle stored
-   * in the display.
-   * TODO: make it a weak reference
-   */
+  /* This is a way to access the real handle
+   * stored in the display.
+   * TODO: make it a weak reference */
   GdkWin32Display *display;
 
-  /* A copy of the "destoyable" attribute of the handle */
+  /* A copy of the "destoyable" attribute of
+   * the handle */
   gboolean         readonly_destroyable;
 };
 
@@ -231,22 +225,16 @@ gdk_win32_hcursor_class_init (GdkWin32HCursorClass *klass)
   object_class->set_property = gdk_win32_hcursor_set_property;
 
   hcursor_props[PROP_DISPLAY] =
-      g_param_spec_object ("display",
-                           P_("Display"),
-                           P_("The display that will use this cursor"),
+      g_param_spec_object ("display", NULL, NULL,
                            GDK_TYPE_DISPLAY,
                            G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 
   hcursor_props[PROP_HANDLE] =
-      g_param_spec_pointer ("handle",
-                            P_("Handle"),
-                            P_("The HCURSOR handle for this cursor"),
+      g_param_spec_pointer ("handle", NULL, NULL,
                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 
   hcursor_props[PROP_DESTROYABLE] =
-      g_param_spec_boolean ("destroyable",
-                            P_("Destroyable"),
-                            P_("Whether calling DestroyCursor() is allowed on this cursor"),
+      g_param_spec_boolean ("destroyable", NULL, NULL,
                             TRUE,
                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 
@@ -388,7 +376,8 @@ hcursor_from_x_cursor (int          i,
   int j, x, y, ofs;
   HCURSOR rv;
   int w, h;
-  guchar *and_plane, *xor_plane;
+  uint8_t *and_plane;
+  uint8_t *xor_plane;
 
   w = GetSystemMetrics (SM_CXCURSOR);
   h = GetSystemMetrics (SM_CYCURSOR);
@@ -412,7 +401,7 @@ hcursor_from_x_cursor (int          i,
 	  for (x = 0; x < cursors[i].width && x < w ; x++, j++)
 	    {
 	      int pofs = ofs + x / 8;
-	      guchar data = (cursors[i].data[j/4] & (0xc0 >> (2 * (j%4)))) >> (2 * (3 - (j%4)));
+              uint8_t data = (cursors[i].data[j/4] & (0xc0 >> (2 * (j%4)))) >> (2 * (3 - (j%4)));
 	      int bit = 7 - (j % cursors[i].width) % 8;
 
 	      if (data)
@@ -858,7 +847,7 @@ static GdkWin32HCursor *
 create_blank_win32hcursor (GdkWin32Display *display)
 {
   int w, h;
-  guchar *and_plane, *xor_plane;
+  uint8_t *and_plane, *xor_plane;
   HCURSOR rv;
 
   w = GetSystemMetrics (SM_CXCURSOR);
@@ -922,7 +911,7 @@ gdk_win32hcursor_create_for_texture (GdkWin32Display *display,
   
   pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
   
-  icon = pixbuf_to_hicon (pixbuf, TRUE, 0, 0);
+  icon = pixbuf_to_hicon (pixbuf, FALSE, x, y);
   
   g_object_unref (pixbuf);
   
@@ -1015,8 +1004,9 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
     RGBQUAD colors[2];
   } bmi;
   HDC hdc;
-  guchar *pixels, *bits;
-  int rowstride, x, y, w, h;
+  uint8_t *pixels, *bits;
+  int x, y, w, h;
+  gsize rowstride;
 
   if (!GDI_CALL (GetIconInfo, (hicon, &ii)))
     return NULL;
@@ -1068,7 +1058,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 		no_alpha = FALSE;
 	      pixels += 4;
 	    }
-	  pixels += (w * 4 - rowstride);
+	  pixels += rowstride - w * 4;
 	}
 
       /* mask */
@@ -1083,7 +1073,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 		  pixels[3] = 255 - bits[(x + y * w) * 4];
 		  pixels += 4;
 		}
-	      pixels += (w * 4 - rowstride);
+	      pixels += rowstride - w * 4;
 	    }
 	}
     }
@@ -1123,7 +1113,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 
       for (y = 0; y < h; y++)
 	{
-	  const guchar *andp, *xorp;
+          const uint8_t *andp, *xorp;
 	  if (bmi.bi.biHeight < 0)
 	    {
 	      andp = bits + bpl*y;
@@ -1157,7 +1147,7 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
 		  xorp++;
 		}
 	    }
-	  pixels += (w * 4 - rowstride);
+	  pixels += rowstride - w * 4;
 	}
     }
 
@@ -1184,8 +1174,8 @@ gdk_win32_icon_to_pixbuf_libgtk_only (HICON hicon,
  */
 
 static HBITMAP
-create_alpha_bitmap (int      size,
-		     guchar **outdata)
+create_alpha_bitmap (int       size,
+                     uint8_t **outdata)
 {
   BITMAPV5HEADER bi;
   HDC hdc;
@@ -1222,9 +1212,9 @@ create_alpha_bitmap (int      size,
 }
 
 static HBITMAP
-create_color_bitmap (int      size,
-		     guchar **outdata,
-		     int      bits)
+create_color_bitmap (int       size,
+                     uint8_t **outdata,
+                     int       bits)
 {
   struct {
     BITMAPV4HEADER bmiHeader;
@@ -1271,8 +1261,9 @@ pixbuf_to_hbitmaps_alpha_winxp (GdkPixbuf *pixbuf,
    * http://www.dotnet247.com/247reference/msgs/13/66301.aspx
    */
   HBITMAP hColorBitmap, hMaskBitmap;
-  guchar *indata, *inrow;
-  guchar *colordata, *colorrow, *maskdata, *maskbyte;
+  const uint8_t *indata;
+  const uint8_t *inrow;
+  uint8_t *colordata, *colorrow, *maskdata, *maskbyte;
   int width, height, size, i, i_offset, j, j_offset, rowstride;
   guint maskstride, mask_bit;
 
@@ -1295,7 +1286,7 @@ pixbuf_to_hbitmaps_alpha_winxp (GdkPixbuf *pixbuf,
   /* MSDN says mask rows are aligned to "LONG" boundaries */
   maskstride = (((size + 31) & ~31) >> 3);
 
-  indata = gdk_pixbuf_get_pixels (pixbuf);
+  indata = gdk_pixbuf_read_pixels (pixbuf);
   rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 
   if (width > height)
@@ -1349,8 +1340,9 @@ pixbuf_to_hbitmaps_normal (GdkPixbuf *pixbuf,
    * http://www.dotnet247.com/247reference/msgs/13/66301.aspx
    */
   HBITMAP hColorBitmap, hMaskBitmap;
-  guchar *indata, *inrow;
-  guchar *colordata, *colorrow, *maskdata, *maskbyte;
+  const uint8_t *indata;
+  const uint8_t *inrow;
+  uint8_t *colordata, *colorrow, *maskdata, *maskbyte;
   int width, height, size, i, i_offset, j, j_offset, rowstride, nc, bmstride;
   gboolean has_alpha;
   guint maskstride, mask_bit;
@@ -1379,7 +1371,7 @@ pixbuf_to_hbitmaps_normal (GdkPixbuf *pixbuf,
   /* MSDN says mask rows are aligned to "LONG" boundaries */
   maskstride = (((size + 31) & ~31) >> 3);
 
-  indata = gdk_pixbuf_get_pixels (pixbuf);
+  indata = gdk_pixbuf_read_pixels (pixbuf);
   rowstride = gdk_pixbuf_get_rowstride (pixbuf);
   nc = gdk_pixbuf_get_n_channels (pixbuf);
   has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
@@ -1481,16 +1473,6 @@ _gdk_win32_texture_to_hicon (GdkTexture *texture)
   return icon;
 }
 
-HICON
-_gdk_win32_pixbuf_to_hcursor (GdkPixbuf *pixbuf,
-			      int        x_hotspot,
-			      int        y_hotspot)
-{
-  return pixbuf_to_hicon (pixbuf, FALSE, x_hotspot, y_hotspot);
-}
-
-
-
 /**
  * gdk_win32_display_get_win32hcursor:
  * @display: (type GdkWin32Display): a `GdkDisplay`
