@@ -27,8 +27,8 @@
 
 #include <gdk/gdkglcontextprivate.h>
 #include <gdk/gdkmemoryformatprivate.h>
-#include <gdk/gdkmemorytextureprivate.h>
 #include <gdk/gdkprofilerprivate.h>
+#include <gdk/gdktexturedownloaderprivate.h>
 #include <gsk/gskdebugprivate.h>
 #include <gsk/gskroundedrectprivate.h>
 
@@ -570,7 +570,10 @@ gsk_gl_command_queue_end_draw (GskGLCommandQueue *self)
   g_assert (self->batches.len > 0);
 
   if (will_ignore_batch (self))
-    return;
+    {
+      self->in_draw = FALSE;
+      return;
+    }
 
   batch = gsk_gl_command_batches_tail (&self->batches);
 
@@ -659,6 +662,10 @@ gsk_gl_command_queue_split_draw (GskGLCommandQueue *self)
 
   g_assert (GSK_IS_GL_COMMAND_QUEUE (self));
   g_assert (self->batches.len > 0);
+
+  if (will_ignore_batch (self))
+    return;
+
   g_assert (self->in_draw == TRUE);
 
   program = self->program_info;
@@ -956,7 +963,7 @@ gsk_gl_command_queue_execute (GskGLCommandQueue    *self,
                               guint                 default_framebuffer)
 {
   G_GNUC_UNUSED guint count = 0;
-  graphene_rect_t scissor_test;
+  graphene_rect_t scissor_test = GRAPHENE_RECT_INIT (0, 0, 0, 0);
   gboolean has_scissor = scissor != NULL;
   gboolean scissor_state = -1;
   guint program = 0;
@@ -999,8 +1006,11 @@ gsk_gl_command_queue_execute (GskGLCommandQueue    *self,
   glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
   glBlendEquation (GL_FUNC_ADD);
 
-  glGenVertexArrays (1, &vao_id);
-  glBindVertexArray (vao_id);
+  if (!gdk_gl_context_get_use_es (self->context))
+    {
+      glGenVertexArrays (1, &vao_id);
+      glBindVertexArray (vao_id);
+    }
 
   vbo_id = gsk_gl_buffer_submit (&self->vertices);
 
@@ -1156,7 +1166,8 @@ gsk_gl_command_queue_execute (GskGLCommandQueue    *self,
     }
 
   glDeleteBuffers (1, &vbo_id);
-  glDeleteVertexArrays (1, &vao_id);
+  if (!gdk_gl_context_get_use_es (self->context))
+    glDeleteVertexArrays (1, &vao_id);
 
   gdk_profiler_set_int_counter (self->metrics.n_binds, n_binds);
   gdk_profiler_set_int_counter (self->metrics.n_uniforms, n_uniforms);
@@ -1356,7 +1367,8 @@ gsk_gl_command_queue_do_upload_texture (GskGLCommandQueue *self,
   GdkGLContext *context;
   const guchar *data;
   gsize stride;
-  GdkMemoryTexture *memtex;
+  GBytes *bytes;
+  GdkTextureDownloader downloader;
   GdkMemoryFormat data_format;
   int width, height;
   GLenum gl_internalformat;
@@ -1391,9 +1403,11 @@ gsk_gl_command_queue_do_upload_texture (GskGLCommandQueue *self,
         }
     }
 
-  memtex = gdk_memory_texture_from_texture (texture, data_format);
-  data = gdk_memory_texture_get_data (memtex);
-  stride = gdk_memory_texture_get_stride (memtex);
+  gdk_texture_downloader_init (&downloader, texture);
+  gdk_texture_downloader_set_format (&downloader, data_format);
+  bytes = gdk_texture_downloader_download_bytes (&downloader, &stride);
+  gdk_texture_downloader_finish (&downloader);
+  data = g_bytes_get_data (bytes, NULL);
   bpp = gdk_memory_format_bytes_per_pixel (data_format);
 
   glPixelStorei (GL_UNPACK_ALIGNMENT, gdk_memory_format_alignment (data_format));
@@ -1423,7 +1437,7 @@ gsk_gl_command_queue_do_upload_texture (GskGLCommandQueue *self,
     }
   glPixelStorei (GL_UNPACK_ALIGNMENT, 4);
 
-  g_object_unref (memtex);
+  g_bytes_unref (bytes);
 }
 
 int
@@ -1439,8 +1453,7 @@ gsk_gl_command_queue_upload_texture (GskGLCommandQueue *self,
 
   g_assert (GSK_IS_GL_COMMAND_QUEUE (self));
   g_assert (!GDK_IS_GL_TEXTURE (texture));
-  g_assert (min_filter == GL_LINEAR || min_filter == GL_NEAREST);
-  g_assert (mag_filter == GL_LINEAR || min_filter == GL_NEAREST);
+  g_assert (mag_filter == GL_LINEAR || mag_filter == GL_NEAREST);
 
   width = gdk_texture_get_width (texture);
   height = gdk_texture_get_height (texture);
@@ -1464,6 +1477,9 @@ gsk_gl_command_queue_upload_texture (GskGLCommandQueue *self,
 
   gsk_gl_command_queue_do_upload_texture (self, texture);
 
+  if (min_filter == GL_LINEAR_MIPMAP_LINEAR)
+    glGenerateMipmap (GL_TEXTURE_2D);
+
   /* Restore previous texture state if any */
   if (self->attachments->textures[0].id > 0)
     glBindTexture (self->attachments->textures[0].target,
