@@ -27,7 +27,6 @@
 #include "gtkadjustmentprivate.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
-#include "gtkintl.h"
 
 
 /**
@@ -137,9 +136,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
    * The value of the adjustment.
    */
   adjustment_props[PROP_VALUE] =
-      g_param_spec_double ("value",
-                           P_("Value"),
-                           P_("The value of the adjustment"),
+      g_param_spec_double ("value", NULL, NULL,
                            -G_MAXDOUBLE, G_MAXDOUBLE,
                            0.0,
                            GTK_PARAM_READWRITE);
@@ -150,9 +147,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
    * The minimum value of the adjustment.
    */
   adjustment_props[PROP_LOWER] =
-      g_param_spec_double ("lower",
-                           P_("Minimum Value"),
-                           P_("The minimum value of the adjustment"),
+      g_param_spec_double ("lower", NULL, NULL,
                            -G_MAXDOUBLE, G_MAXDOUBLE,
                            0.0,
                            GTK_PARAM_READWRITE);
@@ -166,9 +161,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
    * property is nonzero.
    */
   adjustment_props[PROP_UPPER] =
-      g_param_spec_double ("upper",
-                           P_("Maximum Value"),
-                           P_("The maximum value of the adjustment"),
+      g_param_spec_double ("upper", NULL, NULL,
                            -G_MAXDOUBLE, G_MAXDOUBLE,
                            0.0,
                            GTK_PARAM_READWRITE);
@@ -179,9 +172,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
    * The step increment of the adjustment.
    */
   adjustment_props[PROP_STEP_INCREMENT] =
-      g_param_spec_double ("step-increment",
-                           P_("Step Increment"),
-                           P_("The step increment of the adjustment"),
+      g_param_spec_double ("step-increment", NULL, NULL,
                            -G_MAXDOUBLE, G_MAXDOUBLE,
                            0.0,
                            GTK_PARAM_READWRITE);
@@ -192,9 +183,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
    * The page increment of the adjustment.
    */
   adjustment_props[PROP_PAGE_INCREMENT] =
-      g_param_spec_double ("page-increment",
-                           P_("Page Increment"),
-                           P_("The page increment of the adjustment"),
+      g_param_spec_double ("page-increment", NULL, NULL,
                            -G_MAXDOUBLE, G_MAXDOUBLE,
                            0.0,
                            GTK_PARAM_READWRITE);
@@ -209,9 +198,7 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
    * `GtkSpinButton`.
    */
   adjustment_props[PROP_PAGE_SIZE] =
-      g_param_spec_double ("page-size",
-                           P_("Page Size"),
-                           P_("The page size of the adjustment"),
+      g_param_spec_double ("page-size", NULL, NULL,
                            -G_MAXDOUBLE, G_MAXDOUBLE,
                            0.0,
                            GTK_PARAM_READWRITE);
@@ -365,6 +352,21 @@ gtk_adjustment_dispatch_properties_changed (GObject     *object,
     }
 }
 
+static double
+gtk_adjustment_sanitize_value (GtkAdjustment *self,
+                               double         value)
+{
+  GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (self);
+
+  /* don't use CLAMP() so we don't end up below lower if upper - page_size
+   * is smaller than lower
+   */
+  value = MIN (value, priv->upper - priv->page_size);
+  value = MAX (value, priv->lower);
+
+  return value;
+}
+
 /**
  * gtk_adjustment_new:
  * @value: the initial value
@@ -432,6 +434,8 @@ adjustment_set_value (GtkAdjustment *adjustment,
 {
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
+  value = gtk_adjustment_sanitize_value (adjustment, value);
+
   if (priv->value != value)
     {
       priv->value = value;
@@ -510,11 +514,7 @@ gtk_adjustment_set_value_internal (GtkAdjustment *adjustment,
 {
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
-  /* don't use CLAMP() so we don't end up below lower if upper - page_size
-   * is smaller than lower
-   */
-  value = MIN (value, priv->upper - priv->page_size);
-  value = MAX (value, priv->lower);
+  value = gtk_adjustment_sanitize_value (adjustment, value);
 
   if (animate && priv->duration != 0 && priv->clock != NULL)
     {
@@ -838,11 +838,7 @@ gtk_adjustment_configure (GtkAdjustment *adjustment,
   gtk_adjustment_set_page_increment (adjustment, page_increment);
   gtk_adjustment_set_page_size (adjustment, page_size);
 
-  /* don't use CLAMP() so we don't end up below lower if upper - page_size
-   * is smaller than lower
-   */
-  value = MIN (value, upper - page_size);
-  value = MAX (value, lower);
+  value = gtk_adjustment_sanitize_value (adjustment, value);
 
   if (value != priv->value)
     {
