@@ -19,7 +19,7 @@
  * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 /**
@@ -63,13 +63,11 @@
 
 #include "gtkactionhelperprivate.h"
 #include "gtkbuildable.h"
-#include "gtkcheckbutton.h"
 #include "gtkgestureclick.h"
 #include "gtkeventcontrollerkey.h"
 #include "gtkbinlayout.h"
 #include "gtkimage.h"
 #include "gtklabel.h"
-#include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtktypebuiltins.h"
@@ -290,6 +288,9 @@ gtk_button_class_init (GtkButtonClass *klass)
    *
    * This is an action signal. Applications should never connect
    * to this signal, but use the [signal@Gtk.Button::clicked] signal.
+   *
+   * The default bindings for this signal are all forms of the
+   * <kbd>␣</kbd> and <kbd>Enter</kbd> keys.
    */
   button_signals[ACTIVATE] =
     g_signal_new (I_("activate"),
@@ -639,7 +640,7 @@ gtk_button_new_with_label (const char *label)
 
 /**
  * gtk_button_new_from_icon_name:
- * @icon_name: (nullable): an icon name
+ * @icon_name: an icon name
  *
  * Creates a new button containing an icon from the current icon theme.
  *
@@ -671,7 +672,8 @@ gtk_button_new_from_icon_name (const char *icon_name)
  * If characters in @label are preceded by an underscore, they are underlined.
  * If you need a literal underscore character in a label, use “__” (two
  * underscores). The first underlined character represents a keyboard
- * accelerator called a mnemonic. Pressing Alt and that key activates the button.
+ * accelerator called a mnemonic. Pressing <kbd>Alt</kbd> and that key
+ * activates the button.
  *
  * Returns: a new `GtkButton`
  */
@@ -688,7 +690,7 @@ gtk_button_new_with_mnemonic (const char *label)
  *
  * Sets the style of the button.
  *
- * Buttons can has a flat appearance or have a frame drawn around them.
+ * Buttons can have a flat appearance or have a frame drawn around them.
  */
 void
 gtk_button_set_has_frame (GtkButton *button,
@@ -830,8 +832,6 @@ gtk_button_set_label (GtkButton   *button,
           gtk_label_set_use_underline (GTK_LABEL (child), priv->use_underline);
           gtk_label_set_mnemonic_widget (GTK_LABEL (child), GTK_WIDGET (button));
         }
-      if (GTK_IS_CHECK_BUTTON (button))
-        gtk_label_set_xalign (GTK_LABEL (child), 0.0);
 
       gtk_button_set_child (button,  child);
     }
@@ -969,10 +969,6 @@ gtk_button_set_icon_name (GtkButton  *button,
       gtk_image_set_from_icon_name (GTK_IMAGE (priv->child), icon_name);
     }
 
-  gtk_accessible_update_relation (GTK_ACCESSIBLE (button),
-                                  GTK_ACCESSIBLE_RELATION_LABELLED_BY, priv->child, NULL,
-                                  -1);
-
   gtk_button_set_child_type (button, ICON_CHILD);
   g_object_notify_by_pspec (G_OBJECT (button), props[PROP_ICON_NAME]);
 }
