@@ -65,10 +65,11 @@ gtk_column_list_item_factory_setup (GtkListItemFactory *factory,
     {
       GtkColumnViewColumn *column = g_list_model_get_item (columns, i);
 
-      gtk_column_list_item_factory_add_column (self,
-                                               widget,
-                                               column,
-                                               FALSE);
+      if (gtk_column_view_column_get_visible (column))
+        gtk_column_list_item_factory_add_column (self,
+                                                 widget,
+                                                 column,
+                                                 FALSE);
 
       g_object_unref (column);
     }
@@ -102,6 +103,8 @@ gtk_column_list_item_factory_update (GtkListItemFactory *factory,
 {
   GtkListItem *list_item = GTK_LIST_ITEM (item);
   GtkWidget *child;
+  gboolean selectable = TRUE;
+  gboolean activatable = TRUE;
 
   GTK_LIST_ITEM_FACTORY_CLASS (gtk_column_list_item_factory_parent_class)->update (factory, item, unbind, bind, func, data);
 
@@ -109,11 +112,28 @@ gtk_column_list_item_factory_update (GtkListItemFactory *factory,
        child;
        child = gtk_widget_get_next_sibling (child))
     {
+      GtkListItem *cell_item;
+
       gtk_list_item_widget_update (GTK_LIST_ITEM_WIDGET (child),
                                    gtk_list_item_get_position (list_item),
                                    gtk_list_item_get_item (list_item),
                                    gtk_list_item_get_selected (list_item));
+
+       cell_item = gtk_list_item_widget_get_list_item (GTK_LIST_ITEM_WIDGET (child));
+       if (cell_item)
+         {
+           selectable &= gtk_list_item_get_selectable (cell_item);
+           activatable &= gtk_list_item_get_activatable (cell_item);
+        }
     }
+
+  /* This really does not belong here, but doing better
+   * requires considerable plumbing that we don't have now,
+   * and something like this is needed to fix the filechooser
+   * in select_folder mode.
+   */
+  gtk_list_item_set_selectable (list_item, selectable);
+  gtk_list_item_set_activatable (list_item, activatable);
 }
 
 static void
