@@ -110,6 +110,35 @@ typedef enum
   GTK_BASELINE_POSITION_BOTTOM
 } GtkBaselinePosition;
 
+/**
+ * GtkContentFit:
+ * @GTK_CONTENT_FIT_FILL: Make the content fill the entire allocation,
+ *   without taking its aspect ratio in consideration. The resulting
+ *   content will appear as stretched if its aspect ratio is different
+ *   from the allocation aspect ratio.
+ * @GTK_CONTENT_FIT_CONTAIN: Scale the content to fit the allocation,
+ *   while taking its aspect ratio in consideration. The resulting
+ *   content will appear as letterboxed if its aspect ratio is different
+ *   from the allocation aspect ratio.
+ * @GTK_CONTENT_FIT_COVER: Cover the entire allocation, while taking
+ *   the content aspect ratio in consideration. The resulting content
+ *   will appear as clipped if its aspect ratio is different from the
+ *   allocation aspect ratio.
+ * @GTK_CONTENT_FIT_SCALE_DOWN: The content is scaled down to fit the
+ *   allocation, if needed, otherwise its original size is used.
+ *
+ * Controls how a content should be made to fit inside an allocation.
+ *
+ * Since: 4.8
+ */
+typedef enum
+{
+  GTK_CONTENT_FIT_FILL,
+  GTK_CONTENT_FIT_CONTAIN,
+  GTK_CONTENT_FIT_COVER,
+  GTK_CONTENT_FIT_SCALE_DOWN,
+} GtkContentFit;
+
 /**
  * GtkDeleteType:
  * @GTK_DELETE_CHARS: Delete characters.
@@ -823,8 +852,6 @@ typedef enum {
   GTK_LEVEL_BAR_MODE_DISCRETE
 } GtkLevelBarMode;
 
-G_END_DECLS
-
 /**
  * GtkInputPurpose:
  * @GTK_INPUT_PURPOSE_FREE_FORM: Allow any character
@@ -1277,6 +1304,9 @@ typedef enum {
  * @GTK_ACCESSIBLE_ROLE_WIDGET: An interactive component of a graphical user
  *    interface. This is the role that GTK uses by default for widgets.
  * @GTK_ACCESSIBLE_ROLE_WINDOW: An application window.
+ * @GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON: A type of push button
+ *    which stays pressed until depressed by a second activation.
+ *    Since: 4.10
  *
  * The accessible role for a [iface@Accessible] implementation.
  *
@@ -1361,7 +1391,8 @@ typedef enum {
   GTK_ACCESSIBLE_ROLE_TREE_GRID,
   GTK_ACCESSIBLE_ROLE_TREE_ITEM,
   GTK_ACCESSIBLE_ROLE_WIDGET,
-  GTK_ACCESSIBLE_ROLE_WINDOW
+  GTK_ACCESSIBLE_ROLE_WINDOW,
+  GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON
 } GtkAccessibleRole;
 
 /**
@@ -1637,4 +1668,6 @@ typedef enum { /*< prefix=GTK_ACCESSIBLE_SORT >*/
   GTK_ACCESSIBLE_SORT_OTHER
 } GtkAccessibleSort;
 
+G_END_DECLS
+
 #endif /* __GTK_ENUMS_H__ */
