@@ -28,7 +28,6 @@
 #include "gtksettings.h"
 #include "gtkwidget.h"
 #include "gtkdebug.h"
-#include "gtkintl.h"
 #include "gtkcomposetable.h"
 #include "gtkimmoduleprivate.h"
 
@@ -47,8 +46,9 @@
  * `GtkIMContextSimple` reads compose sequences from the first of the
  * following files that is found: ~/.config/gtk-4.0/Compose, ~/.XCompose,
  * /usr/share/X11/locale/$locale/Compose (for locales that have a nontrivial
- * Compose file). The syntax of these files is described in the Compose(5)
- * manual page.
+ * Compose file). A subset of the file syntax described in the Compose(5)
+ * manual page is supported. Additionally, `include "%L"` loads GTK’s built-in
+ * table of compose sequences rather than the locale-specific one from X11.
  *
  * If none of these files is found, `GtkIMContextSimple` uses a built-in table
  * of compose sequences that is derived from the X11 Compose files.
@@ -181,20 +181,6 @@ gtk_im_context_simple_class_init (GtkIMContextSimpleClass *class)
   init_compose_table_async (NULL, NULL, NULL);
 }
 
-static char *
-get_x11_compose_file_dir (void)
-{
-  char * compose_file_dir;
-
-#if defined (X11_DATA_PREFIX)
-  compose_file_dir = g_strdup (X11_DATA_PREFIX "/share/X11/locale");
-#else
-  compose_file_dir = g_build_filename (_gtk_get_datadir (), "X11", "locale", NULL);
-#endif
-
-  return compose_file_dir;
-}
-
 static int
 gtk_compose_table_find (gconstpointer data1,
                         gconstpointer data2)
@@ -322,7 +308,7 @@ gtk_im_context_simple_init_compose_table (void)
         {
           if (g_ascii_strncasecmp (*lang, *sys_lang, strlen (*sys_lang)) == 0)
             {
-              char *x11_compose_file_dir = get_x11_compose_file_dir ();
+              char *x11_compose_file_dir = gtk_compose_table_get_x11_compose_file_dir ();
               path = g_build_filename (x11_compose_file_dir, *lang, "Compose", NULL);
               g_free (x11_compose_file_dir);
               break;
@@ -365,6 +351,8 @@ init_compose_table_thread_cb (GTask            *task,
 
   gtk_im_context_simple_init_compose_table ();
 
+  g_task_return_boolean (task, TRUE);
+
   gdk_profiler_end_mark (before, "im compose table load (thread)", NULL);
 }
 
@@ -664,7 +652,8 @@ no_sequence_matches (GtkIMContextSimple *context_simple,
                                                    gdk_event_get_modifier_state (event),
                                                    FALSE,
                                                    &translated,
-                                                   &translated);
+                                                   &translated,
+                                                   NULL);
 
 	  gtk_im_context_filter_keypress (context, tmp_event);
 	  gdk_event_unref (tmp_event);
@@ -813,7 +802,7 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
   gboolean compose_match;
   guint keyval, state;
 
-  while (priv->compose_buffer[n_compose] != 0 && n_compose < priv->compose_buffer_len)
+  while (n_compose < priv->compose_buffer_len && priv->compose_buffer[n_compose] != 0)
     n_compose++;
 
   keyval = gdk_key_event_get_keyval (event);
