@@ -27,13 +27,15 @@
 
 #include <glib-object.h>
 #include <gdk/gdk.h>
-#include <gdk/gdk-private.h>
+#include <gdk/gdkprivate.h>
+#include <gdk/gdkdebugprivate.h>
 
 #include "gtkcsstypesprivate.h"
 #include "gtktexthandleprivate.h"
 #include "gtkplacessidebarprivate.h"
 #include "gtkeventcontrollerprivate.h"
 #include "gtkwindowgroup.h"
+#include "gtkdebug.h"
 
 G_BEGIN_DECLS
 
@@ -126,25 +128,36 @@ void _gtk_load_dll_with_libgtk3_manifest (const char *dllname);
 
 gboolean        gtk_simulate_touchscreen (void);
 
-void  gtk_set_display_debug_flags (GdkDisplay *display,
-                                   guint       flags);
-guint gtk_get_display_debug_flags (GdkDisplay *display);
-gboolean gtk_get_any_display_debug_flag_set (void);
+void            gtk_set_display_debug_flags        (GdkDisplay    *display,
+                                                    GtkDebugFlags  flags);
+GtkDebugFlags   gtk_get_display_debug_flags        (GdkDisplay    *display);
+gboolean        gtk_get_any_display_debug_flag_set (void);
 
 GBytes *get_emoji_data (void);
 
 #ifdef G_ENABLE_DEBUG
 
-#define GTK_DISPLAY_DEBUG_CHECK(display,type) (gtk_get_any_display_debug_flag_set () && G_UNLIKELY (gtk_get_display_debug_flags (display) & GTK_DEBUG_##type))
-#define GTK_DISPLAY_NOTE(display,type,action) \
-  G_STMT_START { \
-  if (GTK_DISPLAY_DEBUG_CHECK (display,type)) { action; }; \
+#define GTK_DISPLAY_DEBUG_CHECK(display,type)                   \
+  (gtk_get_any_display_debug_flag_set () &&                     \
+   G_UNLIKELY (gtk_get_display_debug_flags (display) & GTK_DEBUG_##type))
+
+#define GTK_DEBUG(type,...)                                     \
+  G_STMT_START {                                                \
+    if (GTK_DEBUG_CHECK (type))                                 \
+      gdk_debug_message (__VA_ARGS__);                          \
+  } G_STMT_END
+
+#define GTK_DISPLAY_DEBUG(display,type,...)                     \
+  G_STMT_START {                                                \
+    if (GTK_DISPLAY_DEBUG_CHECK (display,type))                 \
+      gdk_debug_message (__VA_ARGS__);                          \
   } G_STMT_END
 
 #else
 
 #define GTK_DISPLAY_DEBUG_CHECK(display,type) 0
-#define GTK_DISPLAY_NOTE(display,type,action)
+#define GTK_DISPLAY_DEBUG(display,type,...)
+#define GTK_DEBUG(type,...)
 
 #endif /* G_ENABLE_DEBUG */
 
