@@ -402,7 +402,7 @@ gtk_shortcuts_section_init (GtkShortcutsSection *self)
   gtk_widget_remove_css_class (GTK_WIDGET (self->switcher), "linked");
 
   self->show_all = gtk_button_new_with_mnemonic (_("_Show All"));
-  gtk_widget_hide (self->show_all);
+  gtk_widget_set_visible (self->show_all, FALSE);
   g_signal_connect_swapped (self->show_all, "clicked",
                             G_CALLBACK (gtk_shortcuts_section_show_all), self);
 
