@@ -59,12 +59,10 @@
 #include "gtkprivate.h"
 #include "gtknative.h"
 #include "gtkstack.h"
-#include "gtktreeviewcolumn.h"
 #include "gtkwindowgroup.h"
 #include "gtkrevealer.h"
 #include "gtklayoutmanager.h"
 #include "gtkcssprovider.h"
-#include "gtkstylecontext.h"
 #include "gtkwidgetprivate.h"
 
 
@@ -278,8 +276,6 @@ gtk_inspector_window_init (GtkInspectorWindow *iw)
 
       gtk_stack_add_titled (GTK_STACK (iw->top_stack), widget, name, title);
       gtk_stack_add_named (GTK_STACK (iw->button_stack), button, name);
-      gtk_widget_show (widget);
-      gtk_widget_show (button);
 
       g_free (title);
     }
@@ -588,6 +584,31 @@ gtk_inspector_window_enable_debugging (GtkWindow *window,
   return FALSE;
 }
 
+static void
+force_one_full_redraw (GtkWidget *w)
+{
+  gtk_widget_queue_draw (w);
+  for (w = gtk_widget_get_first_child (w); w; w = gtk_widget_get_next_sibling (w))
+    force_one_full_redraw (w);
+}
+
+static void
+force_full_redraw (GtkInspectorWindow *window)
+{
+  GListModel *toplevels;
+
+  toplevels = gtk_window_get_toplevels ();
+  for (unsigned int i = 0; i < g_list_model_get_n_items (toplevels); i++)
+    {
+      GtkWidget *w = GTK_WIDGET (g_list_model_get_item (toplevels, i));
+
+      if (gtk_widget_get_display (w) == window->inspected_display)
+        force_one_full_redraw (w);
+
+      g_object_unref (w);
+    }
+}
+
 static void
 gtk_inspector_window_class_init (GtkInspectorWindowClass *klass)
 {
@@ -675,6 +696,7 @@ gtk_inspector_window_class_init (GtkInspectorWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, go_up_cb);
   gtk_widget_class_bind_template_callback (widget_class, go_down_cb);
   gtk_widget_class_bind_template_callback (widget_class, go_next_cb);
+  gtk_widget_class_bind_template_callback (widget_class, force_full_redraw);
 }
 
 static GdkDisplay *
@@ -949,7 +971,7 @@ update_go_buttons (GtkInspectorWindow *iw)
       update_go_button (iw->go_next_button,
                         GTK_IS_WIDGET (object) && gtk_widget_get_next_sibling (GTK_WIDGET (object)) != NULL,
                         "Next sibling");
-      gtk_widget_hide (iw->list_position_label);
+      gtk_widget_set_visible (iw->list_position_label, FALSE);
       break;
     case CHILD_KIND_LISTITEM:
       update_go_button (iw->go_down_button, FALSE, NULL);
@@ -959,7 +981,7 @@ update_go_buttons (GtkInspectorWindow *iw)
         char *text = g_strdup_printf ("%u", position);
         gtk_label_set_label (GTK_LABEL (iw->list_position_label), text);
         g_free (text);
-        gtk_widget_show (iw->list_position_label);
+        gtk_widget_set_visible (iw->list_position_label, TRUE);
       }
       break;
     case CHILD_KIND_PROPERTY:
@@ -968,7 +990,7 @@ update_go_buttons (GtkInspectorWindow *iw)
       update_go_button (iw->go_down_button, FALSE, NULL);
       update_go_button (iw->go_previous_button, FALSE, NULL);
       update_go_button (iw->go_next_button, FALSE, NULL);
-      gtk_widget_hide (iw->list_position_label);
+      gtk_widget_set_visible (iw->list_position_label, FALSE);
       break;
     default:
       g_assert_not_reached ();
