@@ -36,6 +36,7 @@
 
 #include "gtk/gtk.h"
 #include "gtk/gtkprinterprivate.h"
+#include "gtk/gtkprivate.h"
 
 #include "gtkprintbackendfile.h"
 
@@ -102,6 +103,7 @@ static GtkPageSetup *       file_printer_get_default_page_size     (GtkPrinter
 
 G_DEFINE_DYNAMIC_TYPE(GtkPrintBackendFile, gtk_print_backend_file, GTK_TYPE_PRINT_BACKEND)
 
+G_MODULE_EXPORT
 void
 g_io_module_load (GIOModule *module)
 {
@@ -115,11 +117,13 @@ g_io_module_load (GIOModule *module)
                                   10);
 }
 
+G_MODULE_EXPORT
 void
 g_io_module_unload (GIOModule *module)
 {
 }
 
+G_MODULE_EXPORT
 char **
 g_io_module_query (void)
 {
@@ -282,8 +286,7 @@ _cairo_write (void                *closure,
 
   error = NULL;
 
-  GTK_NOTE (PRINTING,
-            g_print ("FILE Backend: Writing %u byte chunk to temp file\n", length));
+  GTK_DEBUG (PRINTING, "FILE Backend: Writing %u byte chunk to temp file", length);
 
   while (length > 0) 
     {
@@ -295,8 +298,7 @@ _cairo_write (void                *closure,
         {
           if (error != NULL)
             {
-              GTK_NOTE (PRINTING,
-                        g_print ("FILE Backend: Error writing to temp file, %s\n", error->message));
+              GTK_DEBUG (PRINTING, "FILE Backend: Error writing to temp file, %s", error->message);
 
               g_error_free (error);
             }
@@ -304,8 +306,7 @@ _cairo_write (void                *closure,
 	  return CAIRO_STATUS_WRITE_ERROR;
 	}    
 
-      GTK_NOTE (PRINTING,
-                g_print ("FILE Backend: Wrote %zd bytes to temp file\n", written));
+      GTK_DEBUG (PRINTING, "FILE Backend: Wrote %zd bytes to temp file", written);
       
       data += written;
       length -= written;
@@ -436,8 +437,7 @@ file_write (GIOChannel   *source,
 
       if (error != NULL)
         {
-          GTK_NOTE (PRINTING,
-                    g_print ("FILE Backend: %s\n", error->message));
+          GTK_DEBUG (PRINTING, "FILE Backend: %s", error->message);
 
           g_error_free (error);
         }
@@ -445,8 +445,7 @@ file_write (GIOChannel   *source,
       return FALSE;
     }
 
-  GTK_NOTE (PRINTING,
-            g_print ("FILE Backend: Writing %"G_GSIZE_FORMAT" byte chunk to target file\n", bytes_read));
+  GTK_DEBUG (PRINTING, "FILE Backend: Writing %"G_GSIZE_FORMAT" byte chunk to target file", bytes_read);
 
   return TRUE;
 }
